import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Scanner;

// This class only allows simple graphs (no loops and no multiple edges)
public class SimpleGraph {
	
	/********************************************
	 * 											*
	 *           instance variables				*
	 * 											*
	 ********************************************/
	
	private int numOfFixedVertices;
	private ArrayList<Vertex> vertices;  // vertices of graph
    private ArrayList<Edge> edges;  // edges of graph
    private TutteDrawing tutteDrawing = new TutteDrawing(this);  // the Tutte's drawing of this graph which is a mapping of the vertices of this graph to geometric points
    
    /********************************************
	 * 											*
	 *          getters and setters				*
	 * 											*
	 ********************************************/
    
    public ArrayList<Vertex> getVertices() {
		return vertices;
	}
    
    public int getNumOfFixedVertices() {
		return numOfFixedVertices;
	}
    
    public void setNumOfFixedVertices(int numOfFixedVertices) {
		this.numOfFixedVertices = numOfFixedVertices;
	}
	
	public Vertex addVertex(Vertex vertex) {
		// Does not allow two vertices with one label
		if (vertexExists(vertex))
			return findVertex(vertex);
		else{
			this.vertices.add(vertex);
			return vertex;
		}
	}
	
	public ArrayList<Edge> getEdges() {
		return edges;
	}
	
	public void addEdge(Edge edge) {
    	// Does not allow self-loop
    	if (edge.getV1() != edge.getV2()){
    		// Does not allow multiple edges
    		if (!edgeExists(edge)){
    			edge.getV1().addAdjacentEdge(edge);
    			edge.getV2().addAdjacentEdge(edge);
    	        this.edges.add(edge);
    		}
    	}
	}
	
	public void removeEdge(Edge edge) {
    	if (edgeExists(edge)){
    		// Find the edge
    		edges.remove(findEdge(edge));
    	}
	}
	
	public int n() {
		return vertices.size();
	}
	
	public int m() {
		return edges.size();
	}
	
	/********************************************
	 * 											*
	 *              Constructors				*
	 * 											*
	 ********************************************/
	
	// First constructor
    public SimpleGraph(String fileName){
    	this.vertices = new ArrayList<Vertex>();
    	this.edges = new ArrayList<Edge>();
    	this.readGraphfromFile(fileName);
    }
    
    // Second constructor which is used for random planar graph generation 
    public SimpleGraph(int numOfVertices){
    	this.vertices = new ArrayList<Vertex>();
    	this.edges = new ArrayList<Edge>();
    	for(int i=0; i< numOfVertices; i++)
    		this.vertices.add(new Vertex(i+""));
    }
    
    
    public SimpleGraph(int numOfVertices, int numOfFixedVertices){
    	this(numOfVertices);
    	this.numOfFixedVertices = numOfFixedVertices;
    }
    
    /********************************************
	 * 											*
	 *              Search methods				*
	 * 											*
	 ********************************************/
    
    // Finds a vertex by its label
    public Vertex findVertex(Vertex vertex){
    	return findVertexByLabel(vertex.getLabel());
    }
    
    // Finds a vertex by its label
    public Vertex findVertexByLabel(String label){
    	for (int i=0; i<vertices.size(); i++){
    		if (vertices.get(i).getLabel().equals(label))
    			return vertices.get(i);
    	}
    	return null;
    }
    
    // Finds an edge by its two end points
    public Edge findEdge(Edge edge){
    	return findEdgeByTwoVertices(edge.getV1(), edge.getV2());
    }
    
    // Finds an edge by its two end points
    public Edge findEdgeByTwoVertices(Vertex v1, Vertex v2){
    	for (int i=0; i<edges.size(); i++){
    		if ((edges.get(i).getV1() == v1 && edges.get(i).getV2() == v2) || (edges.get(i).getV2() == v1 && edges.get(i).getV1() == v2))
    			return edges.get(i);
    	}
    	return null;
    }
    
    
    /********************************************
	 * 											*
	 *              Other methods				*
	 * 											*
	 ********************************************/    
    
    // Returns true if a vertex exists or false if it does not exist
    public boolean vertexExists(Vertex vertex){
    	if (findVertex(vertex) != null)
    		return true;
    	else
    		return false;
    }
    
    // Returns true if an edge exists or false if it does not exist
    public boolean edgeExists(Edge edge){
    	if (findEdge(edge) != null)
    		return true;
    	else
    		return false;
    }
    
    /********************************************
	 * 											*
	 *          Input/output methods			*
	 * 											*
	 ********************************************/
    
    // Reads the graph from file 
    public void readGraphfromFile(String fileName){
    	Vertex v1;
    	Vertex v2;
    	try{
			File file = new File(fileName);
			Scanner fileScanner = new Scanner(file);
			
			if (fileScanner.hasNext())
				//GraphTools.numOfFixedVertices = fileScanner.nextInt();
				numOfFixedVertices = fileScanner.nextInt();
			
			for (int i=0; i<numOfFixedVertices; i++)
				addVertex(new Vertex(fileScanner.nextInt()+""));
			
			while (fileScanner.hasNext()) {
				String l1 = fileScanner.nextInt()+"";  // Label of the first vertex in edge
				String l2 = fileScanner.nextInt()+"";  // Label of the second vertex in edge
				int weight = fileScanner.nextInt();  // Weight of the edge
				
				v1 = new Vertex(l1);
				v1 = addVertex(v1);
				
				v2 = new Vertex(l2);
				v2 = addVertex(v2);
				
				addEdge(new Edge(v1,v2,weight));
			}
		}
		catch (Exception e) {
			System.out.printf("Error in the graph input file: "+e.getMessage()+"\n");
			e.printStackTrace();
		}
	}
    
    // writes the graph to file 
    public void writeGraphToFile(String fileName){
    	try{
			File file = new File(fileName+".txt");
			FileWriter outFile = new FileWriter(file);
			PrintWriter out = new PrintWriter(outFile);
			
			out.print(numOfFixedVertices + " ");
			
			for (int i=0; i<numOfFixedVertices; i++){
				out.print(vertices.get(i).getLabel() + " ");
			}
			
			out.println("");
			
			for (Edge e: edges){
				out.println(e.getV1().getLabel()+ " " + e.getV2().getLabel() + " " + e.getWeight());
			}
			
			outFile.close();
			out.flush();
			out.close();
		}
		catch (Exception e) {
			System.out.printf("Error in the graph output file: "+e.getMessage()+"\n");
			e.printStackTrace();
		}
	}
    
    // Produces the adjacency matrix of the graph
    public int[][] adjacencyMatrix(){
		int[][] adjacencyMatrix = new int[this.vertices.size()][this.vertices.size()];
    	for (int i=0; i<vertices.size(); i++){
			for (int j=0; j<vertices.size(); j++) {
				Edge edge = new Edge(vertices.get(i), vertices.get(j));
				if (edgeExists(edge))
					adjacencyMatrix[i][j] = findEdge(edge).getWeight();
				else
					adjacencyMatrix[i][j] = 0;
			}
		}
    	return adjacencyMatrix;
	}
    
    // Prints the adjacency matrix of the graph with weights of the edges. If it indicates 0 it means that there is no edge between the two vertices.
    public void printGraph(){
		int[][] adjacencyMatrix = adjacencyMatrix();
		
		System.out.println("");
		for (int i=0; i<vertices.size(); i++){
			for (int j=0; j<vertices.size(); j++) {
				System.out.printf("%-6d",adjacencyMatrix[i][j]);
			}
			System.out.println("");
		}
	}
    
	public void drawTutteWeightedGraph() {
		GraphGraphics.drawGraph(this, tutteDrawing.calculate());
	}
    
    /********************************************
	 * 											*
	 *               Main method				*
	 * 											*
	 ********************************************/
    
//	public static void main (String[] args) { 
//		SimpleGraph G = new SimpleGraph(args[0]);
//		G.printGraph();
//	}

}
